@@ -3,6 +3,7 @@ from __future__ import annotations
 from typing import TYPE_CHECKING
 
 from dataclasses import dataclass
+from difflib import unified_diff
 import datetime
 import shutil
 import os
@@ -17,7 +18,7 @@ from jinja2 import (
     TemplateNotFound,
 )
 
-from .base import NAMES, AddCommand, RemoveCommand, UpdateCommand, LinkMode
+from .base import NAMES, AddCommand, RemoveCommand, UpdateCommand, LinkMode, LinkCommand
 from .term import FORMAT_MESSAGE
 from .control import (
     RuntimeClosure,
@@ -36,7 +37,7 @@ from .filesystem import (
 )
 
 if TYPE_CHECKING:
-    from typing import Iterable, Optional, ClassVar, Dict
+    from typing import Iterable, ClassVar, Dict
 
     from .base import ModCommand
     from .filesystem import ProjectPath
@@ -189,84 +190,126 @@ class JinjaTemplate:
 
 
 def _link_helper(
-    linker, name, source_path, target_path, force, state
+    op: LinkCommand,
+    linker: FileLinker,
+    name: str,
+    source_path: Path,
+    target_path: Path,
+    state,
 ) -> RuntimeClosure:
-    def _callable():
-        shutil.copyfile(str(source_path), str(target_path))
-        return RuntimeOutput(True)
-
-    def _fail_callable():
-        state["template_modifications"].append(RemoveCommand(linker.mode, name))
-        return RuntimeOutput(False)
-
     message_args = (linker, name, target_path.parent)
+    if op == LinkCommand.show:
+
+        def _callable():
+            return RuntimeOutput(True, output=source_path.read_text())
 
-    if not (source_path.exists() or target_path.exists()):
         return RuntimeClosure(
-            FORMAT_MESSAGE.link(*message_args, mode="fail"), False, _fail_callable
+            FORMAT_MESSAGE.show(linker, name, mode="no-diff"),
+            True,
+            _callable,
         )
-    if target_path.exists() and not force:
+
+    elif op == LinkCommand.diff:
+
+        def _callable():
+            try:
+                from_str = target_path.read_text().strip()
+            except FileNotFoundError:
+                from_str = ""
+            to_str = source_path.read_text().strip()
+
+            return RuntimeOutput(
+                True,
+                output="\n".join(
+                    unified_diff(
+                        from_str.split("\n"),
+                        to_str.split("\n"),
+                        fromfile=f"{target_path}",
+                        tofile=f"{linker.user_str} '{name}'",
+                        lineterm="",
+                    )
+                ),
+            )
+
         return RuntimeClosure(
-            FORMAT_MESSAGE.link(*message_args, mode="exists"), *SUCCESS
+            FORMAT_MESSAGE.show(linker, name, mode="diff"),
+            True,
+            _callable,
         )
-    if target_path.exists():
+
+    else:
+
+        def _callable():
+            shutil.copyfile(str(source_path), str(target_path))
+            return RuntimeOutput(True)
+
+        def _fail_callable():
+            state["template_modifications"].append(RemoveCommand(linker.mode, name))
+            return RuntimeOutput(False)
+
+        if not (source_path.exists() or target_path.exists()):
+            return RuntimeClosure(
+                FORMAT_MESSAGE.link(*message_args, mode="fail"), False, _fail_callable
+            )
+        if target_path.exists() and op == LinkCommand.copy:
+            return RuntimeClosure(
+                FORMAT_MESSAGE.link(*message_args, mode="exists"), *SUCCESS
+            )
+        if target_path.exists() and op == LinkCommand.replace:
+            return RuntimeClosure(
+                FORMAT_MESSAGE.link(*message_args, mode="overwrite"),
+                True,
+                _callable,
+            )
         return RuntimeClosure(
-            FORMAT_MESSAGE.link(*message_args, mode="overwrite"),
+            FORMAT_MESSAGE.link(*message_args, mode="new"),
             True,
             _callable,
         )
-    return RuntimeClosure(
-        FORMAT_MESSAGE.link(*message_args, mode="new"),
-        True,
-        _callable,
-    )
 
 
 def link_name(
+    op: LinkCommand,
     proj_path: ProjectPath,
     state: Dict,
     linker: FileLinker,
     name: str,
-    force: bool = False,
-    target_dir: Optional[Path] = None,
 ) -> RuntimeClosure:
     return _link_helper(
+        op,
         linker,
         name,
         linker.file_path(name).resolve(),
-        (target_dir if target_dir is not None else proj_path.data_dir)
-        / NAMES.rel_data_path(name + linker.suffix, linker.mode),
-        force,
+        proj_path.data_dir / NAMES.rel_data_path(name + linker.suffix, linker.mode),
         state,
     )
 
 
 def link_path(
+    op: LinkCommand,
     proj_path: ProjectPath,
     state: Dict,
     linker: FileLinker,
     source_path: Path,
-    force: bool = False,
 ) -> RuntimeClosure:
     # also check for wrong suffix
     if source_path.suffix != linker.suffix:
         return RuntimeClosure(f"Filetype '{source_path.suffix}' is invalid!", *FAIL)
     return _link_helper(
+        op,
         linker,
         source_path.name,
         source_path.resolve(),
         proj_path.data_dir / NAMES.rel_data_path(source_path.name, linker.mode),
-        force,
         state,
     )
 
 
 @dataclass
 class NameSequenceLinker(AtomicIterable):
+    op: LinkCommand
     mode: LinkMode
     name_list: Iterable[str]
-    force: bool = False
-    target_dir: Optional[Path] = None
 
     def __call__(
         self,
@@ -277,12 +320,11 @@ class NameSequenceLinker(AtomicIterable):
     ) -> Iterable[RuntimeClosure]:
         yield from (
             link_name(
+                self.op,
                 proj_path,
                 state,
                 LINKER_MAP[self.mode],
                 name,
-                force=self.force,
-                target_dir=self.target_dir,
             )
             for name in self.name_list
         )
@@ -290,23 +332,28 @@ class NameSequenceLinker(AtomicIterable):
 
 @dataclass
 class PathSequenceLinker(AtomicIterable):
+    op: LinkCommand
     mode: LinkMode
     path_list: Iterable[Path]
-    force: bool = False
 
     def __call__(
         self, proj_path: ProjectPath, template_dict: Dict, state: Dict, temp_dir: Path
     ) -> Iterable[RuntimeClosure]:
         yield from (
-            link_path(proj_path, state, LINKER_MAP[self.mode], path, force=self.force)
+            link_path(
+                self.op,
+                proj_path,
+                state,
+                LINKER_MAP[self.mode],
+                path,
+            )
             for path in self.path_list
         )
 
 
 @dataclass
 class TemplateDictLinker(AtomicIterable):
-    force: bool = False
-    target_dir: Optional[Path] = None
+    op: LinkCommand = LinkCommand.copy
 
     def __call__(
         self,
@@ -317,10 +364,9 @@ class TemplateDictLinker(AtomicIterable):
     ) -> Iterable[RuntimeClosure]:
         for mode in LinkMode:
             yield from NameSequenceLinker(
+                self.op,
                 mode,
                 template_dict[NAMES.convert_mode(mode)],
-                force=self.force,
-                target_dir=self.target_dir,
             )(proj_path, template_dict, state, temp_dir)
 
 
